using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using WorkFlowCheck.Common.DTO;

namespace WorkFlowCheck.MAUI.Services.Interfaces
{
    public interface ISyncService
    {
        Task SyncDatas(Action<double, string> reportProgress);
        Task SyncCheckPoints_Down(Action<double, string> reportProgress);
        Task SyncCheckPoints_Up(Action<double, string> reportProgress);
        Task SyncLocations_Down(Action<double, string> reportProgress);
        Task SyncEquipments_Down(Action<double, string> reportProgress);

        Task SyncCheckListTemplates_Down(Action<double, string> reportProgress);

        Task SyncCheckListHeader_Down(Action<double, string> reportProgress);
        Task<ApiResponseDTO<string>> SyncCheckListHeader_Up(int Id, Action<double, string> reportProgress);
    }
}
